package com.baidu.translate.asr.demo;

import android.app.Activity;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;

import com.baidu.translate.asr.OnRecognizeListener;
import com.baidu.translate.asr.TransAsrClient;
import com.baidu.translate.asr.TransAsrConfig;
import com.baidu.translate.asr.data.Language;
import com.baidu.translate.asr.data.RecognitionResult;
import com.baidu.translate.asr.demo.adapter.LanguageAdapter;

public class MainActivity extends Activity implements View.OnTouchListener {
    private static final String TAG = "MainActivity";

    private RadioGroup ttsTypeGroup;
    private CheckBox partialCheck;
    private CheckBox autoTtsCheck;
    private TextView resultText;
    private Spinner fromSpinner;
    private Spinner toSpinner;

    private LanguageAdapter fromAdapter;
    private LanguageAdapter toAdapter;

    private Language fromLang = Language.Chinese;
    private Language toLang = Language.English;

    private static final String APP_ID = "";
    private static final String SECRET_KEY = "";

    // 【重要】 - 语音翻译功能关键类
    private TransAsrClient client;
    private TransAsrConfig config;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        initView();
        initLang();
        initClient();
    }

    private void initView() {
        resultText = (TextView) findViewById(R.id.result_text);
        fromSpinner = (Spinner) findViewById(R.id.from_lang_spinner);
        toSpinner = (Spinner) findViewById(R.id.to_lang_spinner);
        partialCheck = (CheckBox) findViewById(R.id.show_partial_check);
        autoTtsCheck = (CheckBox) findViewById(R.id.auto_tts_check);
        ttsTypeGroup = (RadioGroup) findViewById(R.id.eng_tts_type_group);

        findViewById(R.id.hold_speak_btn).setOnTouchListener(this);
    }

    private void initLang() {

        fromAdapter = new LanguageAdapter(this);
        toAdapter = new LanguageAdapter(this);

        // 源语种设置成仅支持语音输入的语种
        fromAdapter.setLanguages(Language.getAsrAvailableLanguages());
        // 目标语种设置成所有语种
        toAdapter.setLanguages(Language.values());

        fromSpinner.setAdapter(fromAdapter);
        toSpinner.setAdapter(toAdapter);

        fromSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                fromLang = fromAdapter.getItem(position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                //                fromLang = Language.Chinese;
            }
        });

        toSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                toLang = toAdapter.getItem(position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                //                toLang = Language.English;
            }
        });
        toSpinner.setSelection(1);
    }

    private void initClient() {
        // 【重要】语音翻译配置类
        // appId及私钥，可在API平台 管理控制台 查看
        config = new TransAsrConfig(APP_ID, SECRET_KEY);

        // 构造client
        client = new TransAsrClient(this, config);
        // 设置回调
        client.setRecognizeListener(new OnRecognizeListener() {
            @Override
            public void onRecognized(int resultType, @NonNull RecognitionResult result) {
                if (resultType == OnRecognizeListener.TYPE_PARTIAL_RESULT) { // 中间结果
                    Log.d(TAG, "中间识别结果：" + result.getAsrResult());
                    resultText.append(getString(R.string.partial_update_title, result.getAsrResult()));
                    resultText.append("\n");

                } else if (resultType == OnRecognizeListener.TYPE_FINAL_RESULT) { // 最终结果
                    if (result.getError() == 0) { // 表示正常，有识别结果
                        Log.d(TAG, "最终识别结果：" + result.getAsrResult());
                        Log.d(TAG, "翻译结果：" + result.getTransResult());
                        // 语音识别结果
                        resultText.append(getString(R.string.asr_result_title, result.getAsrResult()));
                        resultText.append("\n");
                        // 翻译结果
                        resultText.append(getString(R.string.trans_result_title, result.getTransResult()));
                        resultText.append("\n");
                    } else { // 翻译出错
                        Log.d(TAG, "语音翻译出错 错误码：" + result.getError() + " 错误信息：" + result.getErrorMsg());
                        resultText.append(getString(R.string.trans_asr_error, result.getError(), result.getErrorMsg()));
                        resultText.append("\n");
                    }
                }
            }

        });
    }

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        switch (event.getActionMasked()) {
            case MotionEvent.ACTION_DOWN:
                v.setPressed(true);
                startRecognize();
                break;
            case MotionEvent.ACTION_UP:
                v.setPressed(false);
                stopRecognize();
                break;

            default:
                break;
        }
        return true;
    }

    private void startRecognize() {
        Log.d(TAG, "开始语音识别");
        resultText.setText("");

        // ===== 以下是根据选项进行配置，一般情况下，普通开发者的配置是不变的。======
        // 是否回调中间结果（可用于连续上屏），默认为false
        config.setPartialCallbackEnabled(partialCheck.isChecked());
        // 是否自动播报译文结果（仅支持11种语言），默认为true
        config.setAutoPlayTts(autoTtsCheck.isChecked());
        // 英语 英式发音、美式发音配置
        if (ttsTypeGroup.getCheckedRadioButtonId() == R.id.eng_tts_uk_radio) {
            config.setTtsEnglishType(TransAsrConfig.TTS_ENGLISH_TYPE_UK);
        } else {
            config.setTtsEnglishType(TransAsrConfig.TTS_ENGLISH_TYPE_US);
        }
        // 设置开始识别提示音
        config.setRecognizeStartAudioRes(R.raw.bdspeech_recognition_start);
        // 重新设置一下config。
        client.setConfig(config);
        // ======================== 配置结束================================

        // 【重要】开始语音识别
        client.startRecognize(fromLang, toLang);
    }

    private void stopRecognize() {
        Log.d(TAG, "语音识别结束");
        // 【重要】停止语音识别（有回调）
        client.stopRecognize();

        // 【重要】取消语音识别（没有回调）
        // client.cancelRecognize();
    }

}
